package id.co.sangkuriang.inovasi.daerah.master.controller

import id.co.sangkuriang.inovasi.daerah.master.model.request.ApplicationCategoryRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.DeleteDataRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.PaginationRequest
import id.co.sangkuriang.inovasi.daerah.master.service.ApplicationCategoryService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.servlet.http.HttpServletRequest
import javax.validation.Valid

@RestController
@RequestMapping(path = ["application-category"])
class ApplicationCategoryController {

    @Autowired
    private lateinit var service: ApplicationCategoryService

    @GetMapping(value = [""], produces = ["application/json"])
    fun getAll(page: PaginationRequest): ResponseEntity<*>? {
        return service.getAll(page)
    }

    @GetMapping(value = ["{id}"], produces = ["application/json"])
    fun getDetail(@PathVariable id: String): ResponseEntity<*>? {
        return service.getDetail(id)
    }

    @PostMapping(value = [""], produces = ["application/json"])
    fun saveData(
        @Valid @ModelAttribute request: ApplicationCategoryRequest,
        httpServletRequest: HttpServletRequest
    ): ResponseEntity<*>? {
        return service.saveData(request, httpServletRequest)
    }

    @PutMapping(value = ["{id}"], produces = ["application/json"])
    fun updateData(
        @PathVariable id: String,
        @Valid @ModelAttribute request: ApplicationCategoryRequest,
        httpServletRequest: HttpServletRequest
    ): ResponseEntity<*>? {
        return service.updateData(id, request, httpServletRequest)
    }

    @DeleteMapping(value = [""], produces = ["application/json"])
    fun deleteData(@Valid @RequestBody request: DeleteDataRequest): ResponseEntity<*>? {
        return service.deleteData(request)
    }
}
