package id.co.sangkuriang.inovasi.daerah.master.service.impl

import id.co.sangkuriang.inovasi.daerah.master.constant.UPLOAD_ICON_APPLICATION_CATEGORY_DIR
import id.co.sangkuriang.inovasi.daerah.master.constant.UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR
import id.co.sangkuriang.inovasi.daerah.master.constant.VALIDATOR_MSG_HAS_USED
import id.co.sangkuriang.inovasi.daerah.master.exception.BadRequestException
import id.co.sangkuriang.inovasi.daerah.master.helpers.responseCreated
import id.co.sangkuriang.inovasi.daerah.master.helpers.responseNotFound
import id.co.sangkuriang.inovasi.daerah.master.helpers.responseSuccess
import id.co.sangkuriang.inovasi.daerah.master.model.request.ApplicationCategoryRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.DeleteDataRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.PaginationRequest
import id.co.sangkuriang.inovasi.daerah.master.pojo.ApplicationCategory
import id.co.sangkuriang.inovasi.daerah.master.repository.ApplicationCategoryRepository
import id.co.sangkuriang.inovasi.daerah.master.service.ApplicationCategoryService
import id.co.sangkuriang.inovasi.daerah.master.utility.deleteFile
import id.co.sangkuriang.inovasi.daerah.master.utility.uploadFile
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.domain.Page
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Pageable
import org.springframework.data.domain.Sort
import org.springframework.http.ResponseEntity
import org.springframework.stereotype.Service
import java.util.*
import javax.servlet.http.HttpServletRequest

@Suppress("NAME_SHADOWING")
@Service
class ApplicationCategoryServiceImpl : ApplicationCategoryService {

    @Autowired
    private lateinit var repo: ApplicationCategoryRepository

    override fun getAll(page: PaginationRequest): ResponseEntity<*>? {
        return try {
            val page: Page<ApplicationCategory?>? = if (page.enablePage!! == 1) {
                var direction: Sort.Direction = Sort.Direction.ASC
                if (page.sort?.lowercase(Locale.getDefault()).equals("desc")) {
                    direction = Sort.Direction.DESC
                }
                repo.getListPagination(
                    param = page.param.toString().lowercase(Locale.getDefault()),
                    pageable = PageRequest.of(page.page!!, page.size!!, Sort.by(direction, page.sortBy))
                )
            } else {
                repo.getListPagination(
                    param = page.param.toString().lowercase(Locale.getDefault()),
                    pageable = Pageable.unpaged()
                )
            }
            responseSuccess(data = page)
        } catch (e: Exception) {
            throw e
        }
    }

    override fun getDetail(id: String): ResponseEntity<*>? {
        try {
            val data = repo.findByIdAndActive(id)
            if (data.isPresent) {
                return responseSuccess(data = data)
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    override fun saveData(
        request: ApplicationCategoryRequest,
        httpServletRequest: HttpServletRequest
    ): ResponseEntity<*>? {
        return try {
            validateRequest(request)
            val save = ApplicationCategory()
            save.name = request.name
            if (request.icon != null) {
                save.icon = uploadFile(request.icon!!, httpServletRequest, UPLOAD_ICON_APPLICATION_CATEGORY_DIR)
            }

            if (request.image != null) {
                save.image = uploadFile(request.image!!, httpServletRequest, UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR)
            }

            responseCreated(data = repo.save(save))
        } catch (e: Exception) {
            throw e
        }
    }

    override fun updateData(
        id: String,
        request: ApplicationCategoryRequest,
        httpServletRequest: HttpServletRequest
    ): ResponseEntity<*>? {
        try {
            val data = repo.findByIdAndActive(id)
            if (data.isPresent) {
                validateRequest(request, id)
                val update = data.get()
                update.name = request.name

                if (request.icon != null) {
                    update.icon?.let { deleteFile(UPLOAD_ICON_APPLICATION_CATEGORY_DIR, it) }
                    update.icon = uploadFile(request.icon!!, httpServletRequest, UPLOAD_ICON_APPLICATION_CATEGORY_DIR)
                }

                if (request.image != null) {
                    update.image?.let { deleteFile(UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR, it) }
                    update.image =
                        uploadFile(request.image!!, httpServletRequest, UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR)
                }

                update.updatedAt = Date()

                return responseSuccess(data = repo.save(update))
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    override fun deleteData(request: DeleteDataRequest): ResponseEntity<*>? {
        try {
            val data = repo.findByIdAndActive(request.id)
            if (data.isPresent) {
                val update = data.get()
                update.updatedAt = Date()
                update.active = false
                repo.save(update)
                return responseSuccess()
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    override fun count(): ResponseEntity<*>? {
        return try {
            responseSuccess(data = repo.getCount())
        } catch (e : Exception) {
            throw e
        }
    }

    private fun validateRequest(request: ApplicationCategoryRequest, id: String? = null) {
        try {
            val listMessage = mutableListOf<Map<String, String>>()
            val checkName = repo.findByName(request.name)

            if (checkName.isPresent) {
                val msgCheckName = mapOf("name" to "Nama ${request.name} $VALIDATOR_MSG_HAS_USED")
                if (id == null) {
                    listMessage.add(msgCheckName)
                } else {
                    if (checkName.get().id != id) {
                        listMessage.add(msgCheckName)
                    }
                }
            }

            if (listMessage.isNotEmpty()) {
                throw BadRequestException(data = listMessage)
            }
        } catch (e: Exception) {
            throw e
        }
    }
}
