package id.co.sangkuriang.inovasi.daerah.master.utility

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import id.co.sangkuriang.inovasi.daerah.master.constant.UPLOAD_ICON_APPLICATION_CATEGORY_DIR
import id.co.sangkuriang.inovasi.daerah.master.constant.UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR
import id.co.sangkuriang.inovasi.daerah.master.pojo.Users
import org.apache.commons.io.FileUtils
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.multipart.MultipartFile
import java.io.File
import java.io.FileOutputStream
import java.net.MalformedURLException
import java.net.URL
import javax.servlet.http.HttpServletRequest

fun getUsersLogin(): Users? {
    return try {
        val principal = SecurityContextHolder.getContext().authentication.principal as Any
        val objectMapper = ObjectMapper()
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
        objectMapper.readValue(principal.toString(), Users::class.java)
    } catch (e: Exception) {
        null
    }
}

@Throws(MalformedURLException::class)
fun getBaseUrl(request: HttpServletRequest): String {
    try {
        val requestURL = URL(request.requestURL.toString())
        val port = if (requestURL.port == -1) "" else ":" + requestURL.port
        return requestURL.protocol + "://" + requestURL.host + port + request.contextPath + "/"
    } catch (e: Exception) {
        throw e
    }
}

fun getFullUrl(request: HttpServletRequest): String {
    val requestURL = StringBuilder(request.requestURL.toString())
    val queryString = request.queryString
    return if (queryString == null) {
        requestURL.toString()
    } else {
        requestURL.append('?').append(queryString).toString()
    }
}

fun getPathUrl(request: HttpServletRequest): String {
    return getFullUrl(request).replace(getBaseUrl(request), "")
        .replaceAfter("?", "")
        .replace("?", "")
        .replaceAfter("/", "")
        .replace("/", "")
}


fun uploadFile(
    multipart: MultipartFile,
    httpServletRequest: HttpServletRequest,
    path: String,
    fileType: String? = "images"
): String {
    var fos: FileOutputStream? = null
    try {
        var dirType = ""
        when (path) {
            UPLOAD_ICON_APPLICATION_CATEGORY_DIR -> {
                dirType = "application-category-icon"
            }
            UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR -> {
                dirType = "application-category-image"
            }
        }

        val fileName =
            System.currentTimeMillis().toString() + "-" + multipart.originalFilename?.trim()?.replace(" ", "-")

        val dir = File(path)
        if (!dir.exists()) {
            dir.mkdirs()
        }

        val convFile = File(dir.absolutePath + File.separator + fileName)
        fos = FileOutputStream(convFile)
        fos.write(multipart.bytes)
        fos.close()

        return getBaseUrl(httpServletRequest) + "file/$fileType?type=$dirType&fileName=" + fileName
    } catch (e: Exception) {
        throw RuntimeException("Could not store the file. Error: " + e.message)
    } finally {
        fos?.close()
    }
}

fun deleteFile(fileDir: String, file: String?) {
    try {
        val delete = File(fileDir + File.separator + file?.substringAfterLast("fileName="))
        if (delete.exists()) {
            FileUtils.forceDelete(delete)
        }
    } catch (e: Exception) {
        throw e
    }
}
